from unittest.mock import patch

import pytest
from infoblox_client import objects

from gso.products.product_types.router import Router
from test.workflows import (
    assert_complete,
    assert_lso_success,
    extract_state,
    run_workflow,
)


@pytest.mark.workflow()
@patch("gso.services.infoblox.find_host_by_fqdn")
@patch("gso.services.lso_client.execute_playbook")
@patch("gso.services.netbox_client.NetboxClient.get_device_by_name")
@patch("gso.services.librenms_client.LibreNMSClient.get_device")
def test_validate_nokia_router_success(
    mock_get_librenms_device,
    mock_get_device_by_name,
    mock_execute_playbook,
    mock_find_host_by_fqdn,
    nokia_router_subscription_factory,
    faker,
    data_config_filename,
    geant_partner,
):
    #  Run workflow
    subscription_id = nokia_router_subscription_factory()
    mock_fqdn = Router.from_subscription(subscription_id).router.router_fqdn
    mock_v4 = faker.ipv4()
    mock_find_host_by_fqdn.return_value = objects.HostRecord(
        connector=None,
        aliases=[mock_fqdn],
        comment=subscription_id,
        ipv4addrs=[
            objects.IPv4(
                ipv4addr=str(mock_v4),
                configure_for_dhcp=False,
                mac="00:00:00:00:00:00",
                ip=str(mock_v4),
                host=f"lo0.{mock_fqdn}",
            ),
        ],
        name=mock_fqdn,
    )

    initial_router_data = [{"subscription_id": subscription_id}]
    result, process_stat, step_log = run_workflow("validate_router", initial_router_data)

    state = extract_state(result)
    subscription_id = state["subscription_id"]

    for _ in range(2):
        result, step_log = assert_lso_success(result, process_stat, step_log)

    assert_complete(result)

    state = extract_state(result)
    subscription = Router.from_subscription(subscription_id)

    assert subscription.status == "active"
    assert mock_execute_playbook.call_count == 2
    assert mock_find_host_by_fqdn.call_count == 1
    assert mock_get_device_by_name.call_count == 1
    assert mock_get_librenms_device.call_count == 1
    assert "ipam_warning" not in state


@pytest.mark.workflow()
def test_validate_juniper_router_success(
    juniper_router_subscription_factory,
    faker,
    data_config_filename,
    geant_partner,
):
    #  Run workflow
    subscription_id = juniper_router_subscription_factory()

    initial_router_data = [{"subscription_id": subscription_id}]
    result, _, _ = run_workflow("validate_router", initial_router_data)

    assert_complete(result)
    state = extract_state(result)
    subscription = Router.from_subscription(subscription_id)

    assert subscription.status == "active"
    assert "ipam_warning" not in state
